-- functions to get bosses and loot from our tables

local _, YssInstanceMaps = ...

local loot = YssInstanceMaps["Loot"]

local function pathExists(t, k, ...) -- recursive function to check if a table path exists
	if k then
		if t[k] then
			return pathExists(t[k], ...)
		else
			return false
		end
	else
		return true
	end
end

local function splitBosses(list) -- splits the boss string into a table
	local bosses = {}
	for boss in list:gmatch("([^@]+)") do
		bosses[#bosses+1] = boss
	end
	return bosses
end

local bosscache = {}
setmetatable(bosscache, {__mode = "kv"})
function YssInstanceMaps:GetBosses(instanceType, instance) -- returns all the main bosses in an instance (does not return subbosses)
	local key = instanceType..instance
	if bosscache[key] then return bosscache[key] end
	if pathExists(loot, instanceType, instance) then
		bosscache[key] = splitBosses(loot[instanceType][instance].BossList)
		return bosscache[key]
	else
		bosscache[key] = false
		return false
	end
end

local multibosscache = {}
setmetatable(multibosscache, {__mode = "kv"})
function YssInstanceMaps:IsMultiBoss(instanceType, instance, boss) -- checks if a boss is comprised of subbosses
	local key = instanceType..instance..boss
	if multibosscache[key] then return multibosscache[key] end
	if pathExists(loot, instanceType, instance, boss) and (type(loot[instanceType][instance][boss]) == "string") then
		multibosscache[key] = splitBosses(loot[instanceType][instance][boss])
		return multibosscache[key]
	else
		multibosscache[key] = false
		return false
	end
end

local function getLoot(t) -- parse the loot strings into a usable table
	local loot = {}
	for mode, list in pairs(t) do
		loot[mode] = {}
		for itemstring in list:gmatch("([^,]+)") do
			local id, value = itemstring:match("^([^:]+):(.+)$")
			id, value = tonumber(id) or tonumber(itemstring), tonumber(value) or true
			loot[mode][id] = value
		end
	end
	return loot
end

local function multiBossLoot(instanceType, instance, subBosses) -- combine loot of all subbosses
	local loottable = {}
	for i, boss in ipairs(subBosses) do
		local temploot = getLoot(loot[instanceType][instance][boss])
		for mode, lt in pairs(temploot) do
			loottable[mode] = lt[mode] or {}
			for id in pairs(lt) do -- strip percentages since they tend to be useless here (may have to revert that)
				loottable[mode][id] = true
			end
		end
	end
	return loottable
end

local lootCache = {}
setmetatable(lootCache, {__mode = "kv"})
function YssInstanceMaps:GetLoot(instanceType, instance, boss) -- get the raw loot of a boss
	local key = instanceType..instance..boss
	if lootCache[key] then return lootCache[key] end
	if pathExists(loot, instanceType, instance, boss) then
		local mBoss = self:IsMultiBoss(instanceType, instance, boss)
		if mBoss then
			lootCache[key] = multiBossLoot(instanceType, instance, mBoss)
			return lootCache[key]
		else
			lootCache[key] = getLoot(loot[instanceType][instance][boss])
			return lootCache[key]
		end
	else
		lootCache[key] = false
		return false
	end
end

local isBossCache = {}
setmetatable(isBossCache, {__mode = "kv"})
function YssInstanceMaps:IsBoss(instanceType, instance, boss) -- check if a boss exists
	local key = instanceType..instance..boss
	if isBossCache[key] then return isBossCache[key] end
	if pathExists(loot, instanceType, instance, boss) then
		isBossCache[key] = true
		return true
	else
		isBossCache[key] = false
		return false
	end

end
